/* Win32 GUI app skeleton */

#include <windows.h>

#include "fractroam.h"

LRESULT CALLBACK WndProc( HWND hWnd, UINT messg,
								WPARAM wParam, LPARAM lParam );


char szProgName[] = "WinFractRoam 2a"; /* main window title */

HBITMAP hbit; // bitmap
HDC memdc;
HDC myhdc;
HWND myhwnd;

BOOL ready=FALSE,once=FALSE;

void paint(HWND);

void print() {
	int width=(fract_size_get()).width;
	int height=(fract_size_get()).height;
	short* fractal=fract_get();
	int x=0,y=0,i;
	for (y=0,i=0;y<DEFAULT_FRACT_HEIGHT;y++) 
		for (x=0;x<DEFAULT_FRACT_WIDTH;x++,i+=3)
			SetPixelV(memdc,x,y,RGB(fractal[i],fractal[i+1],fractal[i+2]));
	InvalidateRect(myhwnd,NULL,0); //right way to do it, ish... 0==don't erase
}
								
int WINAPI WinMain( HINSTANCE hInst, 	/*Win32 entry-point routine */
					HINSTANCE hPreInst, 
					LPSTR lpszCmdLine, 
					int nCmdShow )
{
	HWND hWnd;
	MSG lpMsg;
	WNDCLASS wc;

	if( !hPreInst )			/*set up window class and register it */
	{
		wc.lpszClassName 	= szProgName;
		wc.hInstance 		= hInst;
		wc.lpfnWndProc		= WndProc;
		wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
		wc.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
		wc.lpszMenuName		= NULL;
		wc.hbrBackground	= GetStockObject( WHITE_BRUSH );
		wc.style			= 0;
		wc.cbClsExtra		= 0;
		wc.cbWndExtra		= 0;

		if( !RegisterClass( &wc ) )
			return FALSE;
	}


	hWnd = CreateWindow( 	szProgName,							/* now create the window */
							szProgName,
							WS_OVERLAPPEDWINDOW,
							CW_USEDEFAULT, // X
							CW_USEDEFAULT, // Y
							DEFAULT_FRACT_WIDTH,
							DEFAULT_FRACT_HEIGHT,
							(HWND)NULL, //handle to "parent"
							(HMENU)NULL, // no menu
							(HANDLE)hInst, // handle to "us"
							(LPSTR)NULL		);

	ShowWindow(hWnd, nCmdShow );
	UpdateWindow( hWnd );
	
	while( GetMessage( &lpMsg, NULL, 0, 0 ) )					/* begin the message loop */
	{
		TranslateMessage( &lpMsg );
		DispatchMessage( &lpMsg );
	}
	return( lpMsg.wParam);
}

void quit(HWND hwnd) {
	KillTimer(hwnd,1);
  	PostQuitMessage( 0 );

}

void tick_event() {
    if (!ready) return;
	ready=FALSE;
	fract_plot();
	ready=TRUE;
	print();
}

void paint(HWND hWnd) {
	HDC hdc;
	PAINTSTRUCT pstruct;
	if (ready) {
		hdc = BeginPaint(hWnd, &pstruct ); /* prepare window for painting*/
		BitBlt(myhdc,0,0,DEFAULT_FRACT_WIDTH,DEFAULT_FRACT_HEIGHT,memdc,0,0,SRCCOPY);
		EndPaint(hWnd, &pstruct );	/* stop painting */
		UpdateWindow(hWnd);
		//more properly call invalidaterect() after a print?
	}
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT messg,				/*callback procedure */
								WPARAM wParam, LPARAM lParam )
{
	HDC hdc; 				/* handle to the device context */
	char coordinates[360];
	switch(messg)
	{
		case WM_TIMER: // stuff
			fract_update();
			tick_event();
		break;

		case WM_LBUTTONDOWN:
			fract_center(LOWORD(lParam),DEFAULT_FRACT_HEIGHT-HIWORD(lParam));
			tick_event;
		break;
		
		case WM_RBUTTONDOWN:
			fract_pilot_toggle();
		break;
		
		case WM_KEYDOWN:
			switch(wParam) {
				case 81: quit(hWnd); break; // q
				case VK_LEFT:
				case 72: fract_move( 0,-1, 0); tick_event(); break; // h
				case VK_RIGHT:
				case 76: fract_move( 0, 1, 0); tick_event(); break; // l
				case VK_DOWN:
				case 74: fract_move(-1, 0, 0); tick_event(); break; // j
				case VK_UP:
				case 75: fract_move( 1, 0, 0); tick_event(); break; // k
				case 90: fract_move( 0, 0,-1); tick_event(); break; // z
				case 88: fract_move( 0, 0, 1); tick_event(); break; // x
				case 83: fract_style_toggle(); tick_event(); break; // s
				case VK_SPACE:
				case 65: fract_pilot_toggle(); tick_event(); break; // a
				case 68: // d
					sprintf(coordinates,"x (%f to %f); y (%f to %f)\ncenter: (%f,%f); width: %f",fract_bounds_get().xmin,fract_bounds_get().xmax,fract_bounds_get().ymin,fract_bounds_get().ymax,(fract_bounds_get().xmax+fract_bounds_get().xmin)/2,(fract_bounds_get().ymax+fract_bounds_get().ymin)/2,fract_bounds_get().xmax-fract_bounds_get().ymax);
					KillTimer(hWnd,1);
					MessageBox(hWnd,coordinates,"Coordinates",MB_OK);
					SetTimer(hWnd,1,120,NULL);
					Sleep(500);
				break;
				default: break;
			}
		break;
		
		case WM_CREATE: // we're a new window, do init stuff
			fract_init();
			fract_plot();
			hdc = GetDC(hWnd);
			myhwnd=hWnd;
			myhdc=hdc;
			hbit= CreateCompatibleBitmap(hdc,DEFAULT_FRACT_WIDTH,DEFAULT_FRACT_HEIGHT);
			memdc = CreateCompatibleDC(hdc); // for the bitmap
			SelectObject(memdc,hbit);
			ready=TRUE;
			tick_event(); // just to get things started :)
			SetTimer(hWnd,1,120,NULL); // update baby!
		break;
		
		case WM_PAINT:
			myhwnd=hWnd;
			paint(hWnd);
		break;
		
		case WM_DESTROY:
			quit(hWnd);
		break;
	
		default:
		return( DefWindowProc( hWnd, messg, wParam, lParam ) );
	}

	return( 0L );
}